function estimation(handles)
%ESTIMATION Summary of this function goes here
%   Detailed explanation goes here

index_selected = get(handles.FilterList_menu,'Value');
list = get(handles.FilterList_menu,'String');
filter = list{index_selected};

x0 = get(handles.X0_table, 'data')'; %   , x,vx,y,vy
tau = str2double(get(handles.Tau_edit, 'string'));
Q = get(handles.Q_table, 'data');
R = get(handles.R_table, 'data');


Z  = [x0(1); x0(3)];

Fp = [1 tau; 
      0   1]; %   
  
F_PRD = [         Fp  zeros(2,2); 
         zeros(2, 2)          Fp]; %    

H = [1 0 0 0;
     0 0 1 0];

Gd = [0 0; 
      1 0; 
      0 0; 
      0 1];
  
if nnz(Q) ~= 0
    S_Q = chol(Q);
    [L_Q, D_Q] = ld(Q);
    U_Q = L_Q';
  else
    S_Q = zeros(2, 2);
    U_Q = zeros(2, 2);
    D_Q = zeros(2, 2);
end

if nnz(R) ~= 0
    S_R = chol(R);
    [L_R, D_R] = ld(R);
    U_R = L_R';
else
    S_R = zeros(2, 2);
    U_R = zeros(2, 2);
    D_R = zeros(2, 2);
end

t0 = 0;
xi = x0;
est_x0 = x0; % zeros(4, 1)?
est_xi = est_x0; 
P = eye(4, 4);
S_P = eye(4,4);
U_P = eye(4,4);
D_P = eye(4,4);

OXY_limits = [(est_x0(1) - 1) (est_x0(1) + 1) (est_x0(3) - 1) (est_x0(3) + 1)];
axis(handles.OXY_plot, OXY_limits); 

fid = fopen('log_est.txt','wt');
fprintf(fid,'------------ INITIAL ------------\n\r');
fprintf(fid,'t\tx1^\tx2^\tx3^\tx4^\tz1\tz2\n\r');
fprintf(fid,'%u\t%-6.3f\t%-6.3f\t%-6.3f\t%-6.3f\t%-6.3f\t%-6.3f\n\r', ...
             t0, est_xi(1), est_xi(2), est_xi(3), est_xi(4), Z(1, 1), Z(2, 1));
    
trajectory = get(handles.Trajectory_table,'data');
sections = size(trajectory,1);
    
for m = 1:sections
    type = upper(trajectory{m,1});
    length = trajectory{m,2};
    
    if type == 'S'
        fprintf(fid,'-------------- PRD --------------\n\r');
        fprintf(fid,'t\tx1^\tx2^\tx3^\tx4^\tz1\tz2\n\r');
        
        Bd = zeros(4, 1);
        
        for t = t0 + 1 : t0 + length
            old_est_xi = est_xi;
            [xi, z] = modStep(xi, F_PRD, Bd, Gd, S_Q, H, S_R);
            Z(:, t + 1) = z;
            
            if strcmp(filter, 'Kalman')
                [est_xi, P] = estStepKalman(est_xi, z, P, F_PRD, Bd, Gd, Q, H, R);
            elseif strcmp(filter, 'SRCF')
                [est_xi, S_P] = estStepSRCF(est_xi, z, S_P, F_PRD, Bd, Gd, S_Q, H, S_R);
            elseif strcmp(filter, 'UD')
                [est_xi, U_P, D_P] = estStepUD(est_xi, z, U_P, D_P, F_PRD, Bd, Gd, U_Q, D_Q, H, U_R, D_R);
            end
            
            OXY_limits = estOutput(t, old_est_xi, est_xi, Z, type, OXY_limits, fid, handles);
        end
    else
        r = trajectory{m,3};
        omega = sqrt(est_xi(2)^2 + est_xi(4)^2) / r;
        est_omega = sqrt(est_xi(2)^2 + est_xi(4)^2) / r;
         
        est_Fc = [         cos(est_omega*tau)  est_omega^(-1)*sin(est_omega*tau); 
              -est_omega * sin(est_omega*tau)             cos(est_omega*tau)];
          
          Fc = [         cos(omega*tau)  omega^(-1)*sin(omega*tau); 
              -omega * sin(omega*tau)             cos(omega*tau)];
           
        F_RDO = [         Fc  zeros(2, 2);
                 zeros(2, 2)          Fc];
             
             est_F_RDO = [         est_Fc  zeros(2, 2);
                 zeros(2, 2)          est_Fc];
          
        if type == 'L'
            est_Bd = [(est_xi(1)-est_xi(4)/est_omega)*(1-cos(est_omega*tau));
                      (est_omega*est_xi(1)-est_xi(4))*sin(est_omega*tau);
                  (est_xi(3)+est_xi(2)/est_omega)*(1-cos(est_omega*tau));
                      (est_omega*est_xi(3)+est_xi(2))*sin(est_omega*tau)];
                  
                  Bd = [(xi(1)-xi(4)/omega)*(1-cos(omega*tau));
                      (omega*xi(1)-xi(4))*sin(omega*tau);
                  (xi(3)+xi(2)/omega)*(1-cos(omega*tau));
                      (omega*xi(3)+xi(2))*sin(omega*tau)];
                  
            fprintf(fid,'------------- RDO(L) -------------\n\r');
        elseif type == 'R'
            Bd = [(xi(1)+xi(4)/omega)*(1-cos(omega*tau));
                      (omega*xi(1)+xi(4))*sin(omega*tau);
                  (xi(3)-xi(2)/omega)*(1-cos(omega*tau));
                      (omega*xi(3)-xi(2))*sin(omega*tau)];
                  
                  est_Bd = [(est_xi(1)+est_xi(4)/est_omega)*(1-cos(est_omega*tau));
                      (est_omega*est_xi(1)+est_xi(4))*sin(est_omega*tau);
                  (est_xi(3)-est_xi(2)/est_omega)*(1-cos(est_omega*tau));
                      (est_omega*est_xi(3)-est_xi(2))*sin(est_omega*tau)];
                 
            fprintf(fid,'------------- RDO(R) -------------\n\r');
        end
         
        fprintf(fid,'t\tx1^\tx2^\tx3^\tx4^\tz1\tz2\n\r');

        for t = t0 + 1 : t0 + length
            old_est_xi = est_xi;
            [xi, z] = modStep(xi, F_RDO, Bd, Gd, S_Q, H, S_R);
            Z(:, t + 1) = z;
            
            if strcmp(filter, 'Kalman')
                [est_xi, P] = estStepKalman(est_xi, z, P, est_F_RDO, est_Bd, Gd, Q, H, R);
             elseif strcmp(filter, 'SRCF')
                [est_xi, S_P] = estStepSRCF(est_xi, z, S_P, est_F_RDO, est_Bd, Gd, S_Q, H, S_R);
            elseif strcmp(filter, 'UD')
                [est_xi, U_P, D_P] = estStepUD(est_xi, z, U_P, D_P, est_F_RDO, est_Bd, Gd, U_Q, D_Q, H, U_R, D_R);
            end
            
            OXY_limits = estOutput(t, old_est_xi, est_xi, Z, type, OXY_limits, fid, handles);
        end
    end
    t0 = t;
end
    
fclose(fid);

