function modeling(handles)
%MODELING Summary of this function goes here
%   Detailed explanation goes here

x0 = get(handles.X0_table, 'data')'; %   , x,vx,y,vy
tau = str2double(get(handles.Tau_edit, 'string'));
Q = get(handles.Q_table, 'data');
R = get(handles.R_table, 'data');


Z  = [x0(1); x0(3)];

Fp = [1 tau; 
      0   1]; %   
  
F_PRD = [         Fp  zeros(2,2); 
         zeros(2, 2)          Fp]; %    

H = [1 0 0 0;
     0 0 1 0];

Gd = [0 0; 
      1 0; 
      0 0; 
      0 1];
  
if nnz(Q) ~= 0
    S_Q = chol(Q);
else
    S_Q = zeros(2, 2);
end

if nnz(R) ~= 0
    S_R = chol(R);
else
    S_R = zeros(2, 2);
end

t0 = 0;
xi = x0;

OXY_limits = [(x0(1) - 1) (x0(1) + 1) (x0(3) - 1) (x0(3) + 1)];
axis(handles.OXY_plot, OXY_limits); 

fid = fopen('log.txt','wt');
fprintf(fid,'------------ INITIAL ------------\n\r');
fprintf(fid,'t\tx1\tx2\tx3\tx4\tz1\tz2\n\r');
fprintf(fid,'%u\t%-6.3f\t%-6.3f\t%-6.3f\t%-6.3f\t%-6.3f\t%-6.3f\n\r', ...
t0, xi(1), xi(2), xi(3), xi(4), Z(1, 1), Z(2, 1));
    
trajectory = get(handles.Trajectory_table,'data');
sections = size(trajectory,1);
    
for m = 1:sections
    type = upper(trajectory{m,1});
    length = trajectory{m,2};
    
    if type == 'S'
        fprintf(fid,'-------------- PRD --------------\n\r');
        fprintf(fid,'t\tx1\tx2\tx3\tx4\tz1\tz2\n\r');
        
        Bd = zeros(4, 1);
        
        for t = t0 + 1 : t0 + length
            old_xi = xi;
            [xi, z] = modStep(xi, F_PRD, Bd, Gd, S_Q, H, S_R);
            Z(:, t + 1) = z;
            OXY_limits = modOutput(t, old_xi, xi, Z, type, OXY_limits, fid, handles);
        end
    else
        r = trajectory{m,3};
        omega = sqrt(xi(2)^2 + xi(4)^2) / r;
         
        Fc = [         cos(omega*tau)  omega^(-1)*sin(omega*tau); 
              -omega * sin(omega*tau)             cos(omega*tau)];
           
        F_RDO = [         Fc  zeros(2, 2);
                 zeros(2, 2)          Fc];
          
        if type == 'L'
            Bd = [(xi(1)-xi(4)/omega)*(1-cos(omega*tau));
                      (omega*xi(1)-xi(4))*sin(omega*tau);
                  (xi(3)+xi(2)/omega)*(1-cos(omega*tau));
                      (omega*xi(3)+xi(2))*sin(omega*tau)];
                  
            fprintf(fid,'------------- RDO(L) -------------\n\r');
        elseif type == 'R'
            Bd = [(xi(1)+xi(4)/omega)*(1-cos(omega*tau));
                      (omega*xi(1)+xi(4))*sin(omega*tau);
                  (xi(3)-xi(2)/omega)*(1-cos(omega*tau));
                      (omega*xi(3)-xi(2))*sin(omega*tau)];
                 
            fprintf(fid,'------------- RDO(R) -------------\n\r');
        end
         
        fprintf(fid,'t\tx1\tx2\tx3\tx4\tz1\tz2\n\r');

        for t = t0 + 1 : t0 + length
            old_xi = xi;
            [xi, z] = modStep(xi, F_RDO, Bd, Gd, S_Q, H, S_R);
            Z(:, t + 1) = z;
            OXY_limits = modOutput(t, old_xi, xi, Z, type, OXY_limits, fid, handles);
        end
    end
    t0 = t;
end
    
fclose(fid);

